from enum import Enum


class ComparisonOperators(Enum):
    GT = ">"
    GTE = ">="
    LT = "<"
    LTE = "<="


class SemverComparison(Enum):
    LT = -1
    GT = 1
    EQ = 0


class PackageTypes(Enum):
    SEMVER = "semver"
    GENERIC = "generic"


class PackageCheck(Enum):
    OK = "ok"
    WARN = "warn"
    BLOCK = "block"


class SupportedPackageManagers(Enum):
    JSONNET = "jsonnet"



from typing import Dict, List

from github import Github

from rubrical.enum import PackageCheck
from rubrical.results import PackageCheckResult
from rubrical.utilities import console


def _generate_report(reporting_data: Dict[str, List[PackageCheckResult]]):
    test = """
## [Rubrical](https://github.com/ivanklee86/rubrical) Report

"""

    for package_manager in reporting_data.keys():
        test += f"### {package_manager}\n\n"

        not_ok_results = [
            x for x in reporting_data[package_manager] if x.check != PackageCheck.OK
        ]
        if not_ok_results:
            test += "| File | Dependency | Result |\n"
            test += "|------|------------|--------|\n"

            for result in not_ok_results:
                if result.check == PackageCheck.BLOCK:
                    test += f"| {result.file} | {result.name} | ❌ {result.version_package} <= {result.version_block}, update to > {result.version_warn} |\n"
                elif result.check == PackageCheck.WARN:
                    test += f"| {result.file} | {result.name} | ⚠️ {result.version_package} <= {result.version_warn} |\n"
        else:
            test += "🟢 All dependencies up to date!"

    return test


def report_github(
    access_token: str,
    custom_url: str,
    repository_name: str,
    pr_id: int,
    reporting_data: Dict[str, List[PackageCheckResult]],
    warnings_found: bool,
    blocks_found: bool,
):
    if warnings_found or blocks_found:
        rubrical_report_exists = False

        # Set up Github
        if custom_url:
            g = Github(base_url=f"{custom_url}/api/v3", login_or_token=access_token)
        else:
            g = Github(access_token)
        repo = g.get_repo(repository_name)
        pr = repo.get_pull(pr_id)

        for issue_comment in pr.get_issue_comments():
            if (
                "[Rubrical](https://github.com/ivanklee86/rubrical) Report"
                in issue_comment.body
            ):
                issue_comment.edit(_generate_report(reporting_data))
                rubrical_report_exists = True

        if not rubrical_report_exists:
            pr.create_issue_comment(_generate_report(reporting_data))
    else:
        console.print_message("Clean report, skipping post to Github comment.")



import abc
from pathlib import Path
from typing import Dict, List

from rubrical.package import Package


class BasePackageManager(abc.ABC):
    name: str = ""
    target_file: str = ""
    found_files: Dict[str, str]
    packages: Dict[str, List[Package]]

    def __init__(self) -> None:
        super().__init__()

        self.found_files = {}
        self.packages = {}

    def read_package_manager_files(self, current_folder: Path):
        package_manager_files = current_folder.rglob(self.target_file)

        for package_manager_file in package_manager_files:
            with open(str(package_manager_file), "r") as file:
                self.found_files[
                    str(package_manager_file.relative_to(current_folder))
                ] = file.read()

    def parse_package_manager_files(self):
        for file in self.found_files.items():
            self.parse_package_manager_file(*file)

    @abc.abstractmethod
    def parse_package_manager_file(self, package_file_filename: str, package_file: str):
        pass



from pathlib import Path

import typer
from benedict import benedict

from rubrical.configuration import RubricalConfig
from rubrical.reporters import gh
from rubrical.rubrical import Rubrical
from rubrical.utilities import console

app = typer.Typer()


@app.command()
def rubrical(
    config: Path = typer.Option(Path("rubrical.yaml"), help="Path to configuration"),
    target: Path = typer.Option(Path().absolute(), help="Path to configuration"),
    block: bool = typer.Option(True, "/--no-block", help="Don't fail if blocks found."),
    repository_name: str = typer.Option(
        "", envvar="RUBRICAL_REPOSITORY", help="Repository name for reporting purposes."
    ),
    pr_id: int = typer.Option(
        0, envvar="RUBRICAL_PR_ID", help="PR ID for reporting purposes."
    ),
    gh_access_token: str = typer.Option(
        "",
        envvar="RUBRICAL_GH_TOKEN",
        help="Github access token for reporting.  Presence will enable Github reporting.",
    ),
    gh_custom_url: str = typer.Option(
        "",
        envvar="RUBRICAL_GH_CUSTOM_URL",
        help="Github Enterprise custom url. e.g. https://github.custom.dev",
    ),
):
    """
    A Python CLI to encourage (😅) people to update their dependencies!
    """

    console.print_header("Rubrical starting!", "⚙️ ")

    console.print_message("Loading configuration.", "📃")
    if config.suffix in [".yaml", ".json", ".toml"]:
        configuration = RubricalConfig(**benedict(config, format=(config.suffix[1:])))
    else:
        raise ValueError(
            "Rubrical only supports YAML, JSON, or TOML configuration files"
        )

    rubrical = Rubrical(configuration, target)
    (warnings_found, blocks_found, check_results) = rubrical.check_package_managers()

    if gh_access_token:
        gh.report_github(
            access_token=gh_access_token,
            custom_url=gh_custom_url,
            repository_name=repository_name,
            pr_id=pr_id,
            reporting_data=check_results,
            warnings_found=warnings_found,
            blocks_found=blocks_found,
        )

    if blocks_found and block:
        console.print_error("Blocked dependencies found!", "🛑")
    elif warnings_found:
        console.print_header(
            "Warnings, some dependencies may need updating soon!", "☢️ "
        )
    else:
        console.print_header("All dependencies up to date!", "🟢")


if __name__ == "__main__":
    app()



from typing import List, Optional

import semver
from pydantic import BaseModel, PrivateAttr

from rubrical.enum import PackageCheck, PackageTypes, SemverComparison


class PackageRequirement(BaseModel):
    name: str
    type: Optional[PackageTypes] = PackageTypes.SEMVER
    warn: str
    block: str
    _semver_warn: semver.VersionInfo = PrivateAttr
    _semver_block: semver.VersionInfo = PrivateAttr

    def __init__(self, **data) -> None:
        super().__init__(**data)

        if type == PackageTypes.SEMVER:
            self._semver_warn = semver.VersionInfo.parse(self.warn)
            self._semver_block = semver.VersionInfo.parse(self.block)

    def check_package(self, package) -> PackageCheck:
        if type == PackageTypes.SEMVER:
            package_semver = semver.VersionInfo.parse(package.version)
            warn_signal = SemverComparison.GT != semver.compare(
                package_semver, self._semver_warn
            )
            block_signal = SemverComparison.GT != semver.compare(
                package_semver, self._semver_block
            )
        else:
            warn_signal = self.warn >= package.version
            block_signal = self.block >= package.version

        if block_signal:
            status = PackageCheck.BLOCK
        elif warn_signal:
            status = PackageCheck.WARN
        else:
            status = PackageCheck.OK

        return status


class PackageManager(BaseModel):
    name: str
    packages: List[PackageRequirement]


class RubricalConfig(BaseModel):
    version: int
    blocking_mode: Optional[bool] = True
    package_managers: List[PackageManager]
